#pragma once

#include <Arduino.h>

#include <ArduinoJson.h>

#include <ESP8266WiFi.h>
#include <ESP8266WebServer.h>

#include <SimpleSerialProtocol.h>

#include <StateData.h>

namespace Connection {
    const char* ssid = "Dragonvale";
    const char* password = "Dr4g0nSl4yr";
    const int jsonBufferSize = 1024;

    // Serial Device
    SimpleSerialProtocol controller;

    // Wifi WebServer
    ESP8266WebServer server(80);

    // Serial
    void initSerial();
    void handleSerial();
    void onControllerPacket(Packet packet);

    // Wifi
    void initWifi();
    void handleWifi();
    void sendResponse(DynamicJsonDocument response);

    void onApiStatusRoute();
    void getCurrentStateRoute();
}

// serial
void Connection::initSerial(){
    controller.init(0, 115200);
    controller.sendPacket("controller", "serial_status", "ready");
}

void Connection::handleSerial(){
    controller.receivePackets(onControllerPacket);
}

void Connection::onControllerPacket(Packet packet) {

    if(strcmp(packet.id, "access") == 0) {
        if (strcmp(packet.command, "update_state") == 0) {
            StateData::updateState(packet.data);
        }
    }
}

// wifi
void Connection::initWifi(){
    controller.sendPacket("controller", "wifi_status", "init_connecting");

    // Set wifi config
    WiFi.hostname("KrabbyHome");
    WiFi.begin(ssid, password);

    // check connection status
    while (WiFi.status() != WL_CONNECTED) {
        controller.sendPacket("controller", "wifi_status", "connecting");
        delay(500);
    }

    // send status
    controller.sendPacket("controller", "wifi_status", "connected");
    
    // Send details
    char ipAdress[50];
    WiFi.localIP().toString().toCharArray(ipAdress,50);
    controller.sendPacket("controller", "wifi_details", ipAdress);

    server.on("/api/status", Connection::onApiStatusRoute);
    server.on("/api/currentstate", Connection::getCurrentStateRoute);

    server.begin();
}

void Connection::handleWifi() {
    server.handleClient();
}

void Connection::sendResponse(DynamicJsonDocument response) {
    // Prepare data to return
    char serializedJson[jsonBufferSize];
    serializeJson(response, serializedJson);
    
    // Send response
    server.send(200, "application/json", serializedJson);
}

void Connection::onApiStatusRoute() {
    // Init Json Document
    DynamicJsonDocument response(jsonBufferSize);

    // Insert data
    response["status"] = "success";
    response["health"] = "healthy";
    response["uptime"] = millis();

    // Send response
    sendResponse(response);
}

void Connection::getCurrentStateRoute() {
    // Init Json Document
    DynamicJsonDocument response(jsonBufferSize);

    // Insert data
    response["status"] = "success";
    response["state"] = StateData::state;


    // Send response
    sendResponse(response);
}